package org.gjt.abies;


public class XmlOutput
{
    private StringBuffer sb = new StringBuffer();
    private int indent = 0;
    private Queue tags = new Queue();


    private static final int START = 0;
    private static final int OPENTAG = 1;
    private static final int CHARDATA = 2;
    private int state = START;

    public XmlOutput()
    {
    }


    public XmlOutput start(String tagName)
    {
        if ( state == OPENTAG )
        {
            sb.append('>');
        }
        if ( state != CHARDATA )
        {
            sb.append('\n');
            writeIndent();
        }
        sb.append('<').append(tagName);
        tags.addTail(tagName);
        state = OPENTAG;
        indent++;
        return this;
    }

    public XmlOutput attrib(String attrName, boolean value)
    {
        if ( value )
            return attrib(attrName,"true");
        else
            return attrib(attrName,"false");
    }

    public XmlOutput attrib(String attrName, int value)
    {
        return attrib(attrName,Integer.toString(value));
    }

    public XmlOutput attrib(String attrName, String value)
    {        
        if ( state != OPENTAG )
            throw new IllegalStateException("Attribute given when in state " + state);
        if ( value == null )
            return this;
        sb.append(' ').append(attrName).append("=\"").append(escapeParameter(value)).append('"');
        // state still OPENTAG
        return this;
    }

    public XmlOutput chardata(String cData)
    {
        if (cData == null || cData.length() == 0 )
            return this;
        if ( state == OPENTAG )
            sb.append('>');
        sb.append(escapeCharData(cData));
        state = CHARDATA;
        return this;
    }
    
    public XmlOutput cdata(String cData)
    {
        if (cData == null || cData.length() == 0 )
            return this;
        if ( state == OPENTAG ) {
            sb.append(">");
        }
        
        sb.append("<![CDATA[");
        sb.append(cData);
        sb.append("]]>");
        
        state = CHARDATA;
        return this;
    }

    public XmlOutput end(String tagName)
    {
        if ( !tagName.equals(tags.peekTail()) )
            throw new IllegalArgumentException("Closed tag " + tagName + " instead of "+ tags.peekTail());
        return end();
    }

    public XmlOutput end()
    {
        indent--;
        switch (state)
        {
            case START:
                sb.append('\n');
                writeIndent();
                sb.append("</").append(tags.removeTail()).append('>');
                break;
            case OPENTAG:
                sb.append("/>");
                tags.removeTail();
                break;
            case CHARDATA:
                sb.append("</").append(tags.removeTail()).append('>');
                break;
            default:
                throw new IllegalStateException("Unknown state " + state);
        }
        state = START;
        return this;
    }

    public XmlOutput addRaw(String text,boolean closetag)
    {
        if ( closetag && state == OPENTAG )
        {
            sb.append('>');
        }
        state = CHARDATA;
        sb.append(text);
        return this;
    }

    public String dumpXML()
    {
        return sb.toString();
    }

    public String toString()
    {
        return dumpXML();
    }

//------------------------------------------------------------------------------
    private void writeIndent()
    {
        for ( int i =0; i < indent; i++)
            sb.append(' ');
    }


//------------------------------------------------------------------------------

    public static String escapeParameter(String txt)
    {
        return txt;
    }

    public static String escapeCharData(String txt)
    {
        return txt;
    }

    public static void main(String argv[])
    {
        XmlOutput xml = new XmlOutput();

        xml.start("test").attrib("foo","bar");
         xml.start("inside").chardata("ABC ").end();
         xml.start("inside").attrib("index","1").end();
         xml.start("inside").attrib("index","2").chardata("Text data").end();
         xml.start("inside").attrib("index","3").attrib("second","vvv").end();
        xml.end("test");

        System.out.println(xml);
    }

}
