
package org.gjt.abies.dialog;

import java.io.*;
import java.io.StringReader;
import java.util.*;

import javax.swing.tree.*;
import javax.swing.tree.MutableTreeNode;

import org.gjt.abies.XmlOutput;
import org.gjt.abies.nanogroovy.*;




public class Dialog extends TagBase{
	
	private String entry;
	private ArrayList nodes = new ArrayList();
	
	
	public void parse(XMLElement root) {
		setEntry(root.getStringAttribute("entry"));
		if ( getEntry() == null ) {
			throw new XMLParseException("dialog","Required attribute entry is missing");
		}
		List children = root.getChildren();
		Iterator it = children.iterator();
		while ( it.hasNext() ) {
			XMLElement nodeXml = (XMLElement)it.next();
			Node n = new Node();
			n.parse(nodeXml,this);
			
			getNodes().add(n);
		}
	}
    
    public boolean containsNodeId(String id) {
        for ( int i =0; i < getNodes().size(); i++ ) {
            if ( getNode(i).getId().equals(id))
                return true;
        }
        return false;
    }
    
    public String getId() {
		return "DIALOG";
	}
	
    public void dumpXml(XmlOutput xml) {
    	xml.start("dialog").attrib("entry",getEntry());
        Iterator inodes = getNodes().iterator();
        while ( inodes.hasNext() ) {
        	((Node)inodes.next()).dumpXml(xml);
        }
        xml.end("dialog");
    }
	
	public String toString() {
        XmlOutput xml = new XmlOutput();
        dumpXml(xml);
        return xml.dumpXML();
    }

    
    public Dialog deepClone() {
        try {
            DialogParser dp = new DialogParser();
            dp.parseDialog(new StringReader(this.toString()));
            return dp.dialog;
        } catch (IOException exc) {
            exc.printStackTrace();
            throw new Error("Cloning Dialog failed");
        }
    }
    

	public void insert(MutableTreeNode child, int index) {
        Node node = (Node)child;
        getNodes().add(index,node);
    }



    public void remove(int index) {
        getNodes().remove(index);
    }



    public void remove(MutableTreeNode node) {
        getNodes().remove(node);
    }


    public void setUserObject(Object object) {
        throw new UnsupportedOperationException("Not implemented for Dialog");
    }


    public void removeFromParent() {
        throw new UnsupportedOperationException("Not implemented for Dialog");
    }



    public void setParent(MutableTreeNode newParent) {
        throw new UnsupportedOperationException("Not implemented for Dialog");
    }


    public TreeNode getChildAt(int childIndex) {
        return (TreeNode)getNodes().get(childIndex);
    }



    public int getChildCount() {
        return getNodes().size();
    }


    public TreeNode getParent() {
        return null;
    }



    public int getIndex(TreeNode node) {
        return getNodes().indexOf(node);
    }


    public boolean getAllowsChildren() {
        return true;
    }


    
    public boolean isLeaf() {
        return false;
    }


    
    public Enumeration children() {
        return Collections.enumeration(getNodes());
    }
    
    
	public void appendTooltipProperties(StringBuffer sb) {
		addTooltipProperty(sb,"entry",getEntry());
	}

    public void setEntry(String entry) {
        this.entry = entry;
    }

    public String getEntry() {
        return entry;
    }

    public void setNodes(ArrayList nodes) {
        this.nodes = nodes;
    }

    public ArrayList getNodes() {
        return nodes;
    }
    
    public Node getNode(int index) {
        return (Node)nodes.get(index);
    }
    
    public Node findNode(String id) {
        for ( int i =0; i < nodes.size(); i++ ) {
            Node n = (Node)nodes.get(i);
            if ( id.equals(n.getId())) {
                return n;
            }
        }
        return null;
    }
    
    public static boolean isEmpty(String txt) {
        return (txt == null) || (txt.trim().length() == 0);
    }
    
    public List/*Problem*/ performProblemCheck() {
        ArrayList problems = new ArrayList();
        TreeSet dead = new TreeSet();
        TreeSet all = new TreeSet();
        findDuplicatedNodes(problems);
        for ( int i =0; i < nodes.size(); i++ ) {
            Node n = (Node)nodes.get(i);
            n.findDuplicatedOptions(problems);
            all.add(n.getId());
            dead.add(n.getId());
        }
        for ( int i =0; i < nodes.size(); i++ ) {
            Node n = (Node)nodes.get(i);
            for ( int j =0; j < n.getOptions().size(); j++ ) {
                Option opt = (Option)n.getOptions().get(j);
                if ( isEmpty(opt.getForward()) || !all.contains(opt.getForward())) {
                    problems.add(new Problem.DanglingForwardLink(opt));
                } else {
                    dead.remove(opt.getForward());
                }
                if ( isEmpty(opt.getDisplay()) && !opt.isAuto()) {
                    problems.add(new Problem.MissingDisplay(opt));
                } else if ( !isEmpty(opt.getDisplay()) && opt.isAuto() ) {
                    problems.add(new Problem.AutoDisplayCheck(opt));
                }
            }
        }
        
        Iterator it = dead.iterator();
        while ( it.hasNext() ) {
            String nodeName = (String)it.next();
            problems.add(new Problem.DeadNode(findNode(nodeName)));
        }
        
        
        return problems;
    }
    
    public void findDuplicatedNodes(List problems) {
        TreeSet alreadyReported = new TreeSet();
        for ( int i =0; i < nodes.size(); i++ ) {
            Node n = (Node)nodes.get(i);
            if ( alreadyReported.contains(n.getId())) {
                continue;
            }
            ArrayList duplicates = new ArrayList();
            duplicates.add(n);
            for ( int j =i+1; j < nodes.size(); j++ ) {
                Node potentialDuplicate = (Node)nodes.get(j);
                if ( potentialDuplicate.getId().equals(n.getId())) {
                    duplicates.add(potentialDuplicate);
                }
            }
            if ( duplicates.size() > 1 ) {
                alreadyReported.add(n.getId());
                problems.add(new Problem.DuplicateNodeId((Node[])duplicates.toArray(new Node[duplicates.size()])));
            }
        }
    }
    
}
