package org.gjt.abies.dialog;

import java.io.*;
import java.util.Iterator;


public class DialogToJava extends DialogParser {
	
	
	/**
	 * @param string
	 */
	public void generateJavaFile(String javaClassName, File output) throws IOException {
		FileWriter fw = new FileWriter(output);
		StringBuffer preconditionStream = new StringBuffer();
		StringBuffer choiceStream = new StringBuffer();
		
		fw.write("public class " + javaClassName + "{\n");
		fw.write(" DialogEnvironment env;\n");
		fw.write(" public "+javaClassName+"(DialogEnvironment denv){\n");
		fw.write("  env = denv;\n");
		fw.write(" }\n");
		fw.write(" public String dialogEntry() { return node_" +dialog.getEntry() + "();}\n");
		Iterator it = dialog.getNodes().iterator();
		while ( it.hasNext() ) {
			Node node = (Node)it.next();
			fw.write(" public void node_"+node.getId() + "() {\n");
			if ( node.getExec() != null ) {
				fw.write("  { // exec\n");
				fw.write(node.getExec());
				fw.write("\n  }//exec end\n");
			}
			if ( node.getDisplay() != null ) {
				fw.write("  env.display("+quoteJavaString(node.getDisplay())+");\n");
			}
			
			choiceStream.append("  if ( "+quoteJavaString(node.getId()) +".equals(node) ) {\n");
			
			
			Iterator optI = node.getOptions().iterator();
			while ( optI.hasNext() ) {
				Option opt = (Option)optI.next();
				if ( opt.getPrecondition() != null ) {
					fw.write("  if ( option_"+node.getId() + "_"+opt.getId()+"_pre() ) {\n");
					
					String precond = opt.getPrecondition();
					if ( precond.indexOf("return") < 0) {
						precond = "return " + precond + ";";
					}
					
					preconditionStream.append(" public boolean option_"+node.getId() + "_"+opt.getId()+"_pre() {").append(precond).append("}\n");	
				}
				if ( opt.isAuto() ) {
					fw.write("   choice("+quoteJavaString(node.getId()) + ","+ quoteJavaString(opt.getId()) + "); return;\n");
				} else {
					fw.write("   env.displayOption("+quoteJavaString(node.getId())+","+quoteJavaString(opt.getId()) + ","+quoteJavaString(opt.getDisplay()) + ");\n");
				}
				
				if ( opt.getPrecondition() != null ) {
					fw.write ("  }\n");
				}
				
				choiceStream.append("   if ( "+quoteJavaString(opt.getId())+ ".equals(option) ) {\n");
				if ( opt.getExec() != null ) {
					choiceStream.append("    { // exec\n");
					choiceStream.append(opt.getExec());
					choiceStream.append("\n    }//exec end\n");
				}
				choiceStream.append("    node_" + opt.getForward() + "();\n");
				choiceStream.append("    return;\n");
				choiceStream.append("   }\n");
				
			}
			fw.write(" }\n");
			
			choiceStream.append("  }\n");
		}
		
		fw.write(" public void choice(String node, String option) {\n");
		fw.write(choiceStream.toString());
		fw.write(" }\n");
		
		
		fw.write(preconditionStream.toString());
		fw.write("}\n");
		fw.close();
		
		
	}

	public static void main(String[] args) throws IOException {
		if ( args.length == 0) {
			System.out.println("Usage: " + DialogToJava.class.getName() + " dialog1.xml dialog2.xml ...");
			System.exit(1);
		}
		for ( int i =0; i < args.length; i++ ) {
			DialogToJava dtj = new DialogToJava();
			dtj.parseDialog(new FileReader(args[i]));
			String name = new File(args[i]).getName();
			String base = name.substring(0,name.indexOf('.'));
			dtj.generateJavaFile(base,new File(base+".java"));
		}
	}

	
}
