package org.gjt.abies.dialog;

import java.util.*;

import javax.swing.tree.*;

import org.gjt.abies.XmlOutput;
import org.gjt.abies.nanogroovy.*;

public class Node extends TagBase{

    private String id;
    private String exec;
    private String display;
    private ArrayList options = new ArrayList();
    private Dialog parent;

    public void parse(XMLElement root, Dialog aParent) {
        parent = aParent;
        setId(root.getStringAttribute("id"));
        if (getId() == null) { throw new XMLParseException("node",
                "Missing id for node"); }

        setExec(getSingleChildContent(root, "exec"));
        setDisplay(getSingleChildContent(root, "display"));

        List optionTags = root.getChildren("option");
        Iterator it = optionTags.iterator();
        while (it.hasNext()) {
            Option opt = new Option();
            opt.parse((XMLElement) it.next(), this);
            getOptions().add(opt);
        }
    }
    
    public boolean containsOptionId(String id) {
        for ( int i =0; i < getOptions().size(); i++ ) {
            if ( ((Option)getOptions().get(i)).getId().equals(id))
                return true;
        }
        return false;
    }

    public void dumpXml(XmlOutput xml) {
        xml.start("node").attrib("id", getId());
        if (getExec() != null) {
            xml.start("exec");
            xml.cdata(getExec());
            xml.end();
        }
        if (getDisplay() != null) {
            xml.start("display");
            xml.cdata(getDisplay());
            xml.end();
        }
        Iterator iopt = getOptions().iterator();
        while (iopt.hasNext()) {
            ((Option) iopt.next()).dumpXml(xml);
        }
        xml.end("node");
    }

    public String toString() {
        XmlOutput xml = new XmlOutput();
        dumpXml(xml);
        return xml.dumpXML();
    }



    public void insert(MutableTreeNode child, int index) {
        Option opt = (Option) child;
        getOptions().add(index,opt);
    }

    public void remove(int index) {
        getOptions().remove(index);
    }

    public void remove(MutableTreeNode node) {
        System.out.println("Trying to remove " + node);
        getOptions().remove(node);
    }

    public void setUserObject(Object object) {
        throw new UnsupportedOperationException("Not implemented for Node");
    }

    public void removeFromParent() {
        parent.remove(this);
    }

    public void setParent(MutableTreeNode newParent) {
        parent = (Dialog) newParent;
    }

    public TreeNode getChildAt(int childIndex) {
        return (TreeNode) getOptions().get(childIndex);
    }

    public int getChildCount() {
        return getOptions().size();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.tree.TreeNode#getParent()
     */
    public TreeNode getParent() {
        return parent;
    }

    public int getIndex(TreeNode node) {
        return getOptions().indexOf(node);
    }

    public boolean getAllowsChildren() {
        return true;
    }

    public boolean isLeaf() {
        return false;
    }

    public Enumeration children() {
        return Collections.enumeration(getOptions());
    }

    public void appendTooltipProperties(StringBuffer sb) {
        addTooltipProperty(sb,"id",getId());
        addTooltipProperty(sb,"exec",getExec());
        addTooltipProperty(sb,"display",getDisplay());
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getId() {
        return id;
    }

    public void setExec(String exec) {
        this.exec = exec;
    }

    public String getExec() {
        if ( exec != null) {
            if ( exec.trim().length() == 0 ) {
                exec = null;
            }
        }
        return exec;
    }

    public void setDisplay(String display) {
        this.display = display;
    }

    public String getDisplay() {
        return display;
    }

    public void setOptions(ArrayList options) {
        this.options = options;
    }

    public ArrayList getOptions() {
        return options;
    }
    
    public Option getOption(int index) {
        return (Option)options.get(index);
    }
    
    public void setParent(Dialog parent) {
        this.parent = parent;
    }

    public void findDuplicatedOptions(List problems) {
        TreeSet alreadyReported = new TreeSet();
        for ( int i =0; i < options.size(); i++ ) {
            Option n = (Option)options.get(i);
            if ( alreadyReported.contains(n.getId())) {
                continue;
            }
            ArrayList duplicates = new ArrayList();
            duplicates.add(n);
            for ( int j =i+1; j < options.size(); j++ ) {
                Option potentialDuplicate = (Option)options.get(j);
                if ( potentialDuplicate.getId().equals(n.getId())) {
                    duplicates.add(potentialDuplicate);
                }
            }
            if ( duplicates.size() > 1 ) {
                alreadyReported.add(n.getId());
                problems.add(new Problem.DuplicateOptionId((Option[])duplicates.toArray(new Option[duplicates.size()])));
            }
        }
        
    }

}