
package org.gjt.abies.dialog;

import java.util.Enumeration;

import javax.swing.tree.*;
import javax.swing.tree.MutableTreeNode;

import org.gjt.abies.XmlOutput;
import org.gjt.abies.nanogroovy.*;



public class Option extends TagBase{
	
	private String id;
	private boolean auto;
	private String forward;
	private String display;
	private String precondition;
	private String exec;
    private Node parent;
	
	public void parse(XMLElement root, Node aParent) {
        parent = aParent;
		setId(root.getStringAttribute("id"));
		if ( getId() == null ) {
			throw new XMLParseException("option","Missing id for option");
		}
		setForward(root.getStringAttribute("forward"));
		if ( getForward() == null ) {
			throw new XMLParseException("option","Missing forward for option "+getId());
		}
		
		setAuto(root.getBooleanAttribute("auto","true","false",false));
		
		setPrecondition(getSingleChildContent(root,"precondition"));
		setDisplay(getSingleChildContent(root,"display"));
		/*if ( auto && display != null) {
			throw new XMLParseException("option","Option cannot have auto=true and display set");
		}*/
		setExec(getSingleChildContent(root,"exec"));
		
	}
    
    public String toString() {
        XmlOutput xml = new XmlOutput();
        dumpXml(xml);
        return xml.dumpXML();
    }
    
    
	
	public void dumpXml(XmlOutput xml) {
        xml.start("option").attrib("id",getId()).attrib("forward",getForward()).attrib("auto",isAuto());
        if ( getPrecondition() != null ) {
            xml.start("precondition");
            xml.cdata(getPrecondition());
            xml.end();
        }
        if ( getDisplay() != null ) {
            xml.start("display");
            xml.cdata(getDisplay());
            xml.end();
        }
        if ( getExec() != null ) {
            xml.start("exec");
            xml.cdata(getExec());
            xml.end();
        }
        xml.end("option");
		
	}
    
	
    


	public void removeFromParent() {
        parent.remove(this);
	}
	
	public void remove(int index) {
        throw new UnsupportedOperationException("Not defined for Option");
	}

	public void setUserObject(Object object) {
        throw new UnsupportedOperationException("Not defined for Option");
	}

	public void remove(MutableTreeNode node) {
        throw new UnsupportedOperationException("Not defined for Option");
	}

	public void setParent(MutableTreeNode newParent) {
		parent = (Node)newParent;		
	}

	public void insert(MutableTreeNode child, int index) {
        throw new UnsupportedOperationException("Not defined for Option");
	}

	public int getChildCount() {
		return 0;
	}

	public boolean getAllowsChildren() {
		return false;
	}

	public boolean isLeaf() {
		return true;
	}

	public Enumeration children() {
		return DefaultMutableTreeNode.EMPTY_ENUMERATION;
	}

	public TreeNode getParent() {
		return getNodeParent();
	}
    
    public Node getNodeParent() {
        return parent;
    }

	public TreeNode getChildAt(int childIndex) {
        throw new UnsupportedOperationException("Not defined for Option");
	}

	public int getIndex(TreeNode node) {
        throw new UnsupportedOperationException("Not defined for Option");
	}
    
    public void appendTooltipProperties(StringBuffer sb) {
        addTooltipProperty(sb,"id",getId());
        addTooltipProperty(sb,"precondition",getPrecondition());
        addTooltipProperty(sb,"display",getDisplay());
        addTooltipProperty(sb,"exec",getExec());
        addTooltipProperty(sb,"forward",(isAuto() ? "AUTO:" : "") + getForward());
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getId() {
        return id;
    }

    public void setAuto(boolean auto) {
        this.auto = auto;
    }

    public boolean isAuto() {
        return auto;
    }

    public void setForward(String forward) {
        this.forward = forward;
    }

    public String getForward() {
        return forward;
    }

    public void setDisplay(String display) {
        this.display = display;
    }

    public String getDisplay() {
        return display;
    }

    public void setPrecondition(String precondition) {
        this.precondition = precondition;
    }

    public String getPrecondition() {
        if ( precondition != null ) {
            if ( precondition.trim().length() == 0 ) {
                precondition = null;
            }
        }
        return precondition;
    }

    public void setExec(String exec) {
        this.exec = exec;
    }

    public String getExec() {
        if ( exec != null) {
            if ( exec.trim().length() == 0 ) {
                exec = null;
            }
        }
        return exec;
    }
    
    public void setParent(Node parent) {
        this.parent = parent;
    }
    
    
}
