package org.gjt.abies.dialog;

public abstract class Problem {
    
    public abstract String getDescription();
    public abstract String getSolution();
    public TagBase[] getOffendingTags() {return new TagBase[0]; };
    
    public static class DuplicateNodeId extends Problem {
        Node[] duplicatedNodes;
        public DuplicateNodeId(Node[] theDuplicatedNodes) {
            duplicatedNodes = theDuplicatedNodes;
        }
        public Node[] getDuplicatedNodes() {return duplicatedNodes;}
        public TagBase[] getOffendingTags() {
            return getDuplicatedNodes();
        }
        public String toString() {return "Duplicate node id " + getDuplicatedNodes()[0].getId();}
        public String getDescription() { return "More than one node has same id:" + getDuplicatedNodes()[0].getId();}
        public String getSolution() { return "Rename all but one nodes";}
    }
    
    public static class DuplicateOptionId extends Problem {
        Option[] duplicatedOptions;
        public DuplicateOptionId(Option[] theDuplicatedOptions) {
            duplicatedOptions = theDuplicatedOptions;
        }
        public Option[] getDuplicatedOptions() {return duplicatedOptions;}
        public TagBase[] getOffendingTags() {
            return getDuplicatedOptions();
        }
        public String toString() {return "Duplicate option id " + getDuplicatedOptions()[0].getId();}
        public String getDescription() { return "More than one option in node "+getDuplicatedOptions()[0].getNodeParent().getId()+" has same id:" + duplicatedOptions[0].getId();}
        public String getSolution() { return "Rename all but one option in given node";}
    }
    
    public static class DeadNode extends Problem {
        Node deadNode;
        public DeadNode(Node aDeadNode) {
            deadNode = aDeadNode;
        }
        public Node getDeadNode() {return deadNode;}
        
        public TagBase[] getOffendingTags() {
            return new TagBase[] {getDeadNode()};
        }
        
        public String toString() {
            return "Dead node " + getDeadNode().getId();
        }
        public String getDescription() {
            return "Node " + getDeadNode().getId() + " is not linked from any option in dialog";
        }
        
        public String getSolution() {
            return "Delete this node or create an option pointing to it from live node";
        }
        
    }
    
    public static class AutoDisplayCheck extends Problem {
        Option autoOption;
        public AutoDisplayCheck(Option anAutoOption) {
            autoOption = anAutoOption;
        }
        public Option getAutoOption() {return autoOption;}
        
        public TagBase[] getOffendingTags() {
            return new TagBase[] {getAutoOption()};
        }
        
        public String toString() {
            return "Auto/display conflict in option " + getAutoOption().getNodeParent().getId() + "/" + getAutoOption().getId();  
        }
        
        public String getDescription() {
            return "Option " + getAutoOption().getId() + " at node " + getAutoOption().getNodeParent().getId() + " has both display and auto property set";
        }
        
        
        public String getSolution() {
            return "Option should have either auto forward or display set, not both";
        }
    }
    
    public static class MissingDisplay extends Problem {
        Option nakedOption;
        public MissingDisplay(Option aNakedOption) {
            nakedOption = aNakedOption;
        }
        public Option getNakedOption() {return nakedOption; }
        
        public TagBase[] getOffendingTags() {
            return new TagBase[] {getNakedOption()};
        }
        
        public String toString() {
            return "Missing display in option " + getNakedOption().getNodeParent().getId() + "/" + getNakedOption().getId();  
        }
        
        public String getDescription() {
            return "Option " + getNakedOption().getId() + " at node " + getNakedOption().getNodeParent().getId() + " has no display property";
        }
        
        
        public String getSolution() {
            return "Add display to selected option or make it auto forward if nothing should be displayed and option should be selected automatically";
        }
    }
    
    public static class DanglingForwardLink extends Problem {
        Option blindOption;
        public DanglingForwardLink(Option aBlindOption) {
            blindOption = aBlindOption;
        }
        public Option getBlindOption() {return blindOption; }
        public TagBase[] getOffendingTags() {
            return new TagBase[] {getBlindOption()};
        }
        
        public String toString() {
            return "Dangling forward in option " + getBlindOption().getNodeParent().getId() + "/" + getBlindOption().getId();  
        }
        public String getDescription() { 
            return "Option " + getBlindOption().getId() + " at node " + getBlindOption().getNodeParent().getId() + " has forward pointer which does not indicate valid node";
        }
        
        
        public String getSolution() {
            return "Point option to existing node or create new node with given name";
        }
    }
    
}
