package org.gjt.abies.dialog.editor;

import java.awt.event.*;
import java.beans.PropertyVetoException;
import java.io.File;

import javax.swing.*;
import javax.swing.event.*;

import org.gjt.abies.dialog.Dialog;



public class DialogDocument extends JInternalFrame implements InternalFrameListener, DialogPanelEnvironment {
    
    
    public static final String DOC_FRAME_PROP = "doc.frame";
    
    Editor mainEditor;
    DialogPanel dialogPanel;
    JInternalFrame problemFrame;
    
    
    
	/**
	 * @param title
	 */
	public DialogDocument(Dialog aRoot, File aFile,Editor main) {
		super((aFile!=null ? aFile.getName() : "NewDialog"),true,true,true,true);
        dialogPanel = new DialogPanel(this,aRoot,aFile);
        mainEditor = main;
        
        setBounds(Editor.getPrefBounds(DOC_FRAME_PROP,"0x0x700x500"));
        addComponentListener(new ResizeListener(DOC_FRAME_PROP));
        
        this.setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
        this.addInternalFrameListener(this);

        //TODO: add visual hints for drag/drop transfer
        
        
        JMenuBar jmb = new JMenuBar();
            
            JMenu file = new JMenu(_("File"));
                file.add(createMenuItem(mainEditor.openAction,KeyStroke.getKeyStroke(KeyEvent.VK_O,InputEvent.CTRL_DOWN_MASK)));
                file.addSeparator();
                file.add(createMenuItem(dialogPanel.save,KeyStroke.getKeyStroke(KeyEvent.VK_S,InputEvent.CTRL_DOWN_MASK)));
                file.add(new JMenuItem(dialogPanel.saveAs));
                file.addSeparator();
                file.add(createMenuItem(close,KeyStroke.getKeyStroke(KeyEvent.VK_F4,InputEvent.CTRL_DOWN_MASK)));
            jmb.add(file);
            
            JMenu edit = new JMenu(_("Edit"));
                edit.add(createMenuItem(dialogPanel.undo,KeyStroke.getKeyStroke(KeyEvent.VK_Z,InputEvent.CTRL_DOWN_MASK)));
                edit.add(createMenuItem(dialogPanel.redo,KeyStroke.getKeyStroke(KeyEvent.VK_Y,InputEvent.CTRL_DOWN_MASK)));
                edit.add(createMenuItem(dialogPanel.refresh,KeyStroke.getKeyStroke(KeyEvent.VK_F5,0)));
                edit.addSeparator();
                edit.add(createMenuItem(dialogPanel.deleteSelected,KeyStroke.getKeyStroke(KeyEvent.VK_DELETE,0)));
            jmb.add(edit);
            
            JMenu export = new JMenu(_("Export"));
                export.add(createMenuItem(problemCheck,KeyStroke.getKeyStroke(KeyEvent.VK_F5,InputEvent.CTRL_DOWN_MASK)));
                export.add(dialogPanel.exportAsJava);
                export.add(simpleRun);
            jmb.add(export);
            
        setJMenuBar(jmb);
        
        getContentPane().add(dialogPanel);
        
	}
    
    private static JMenuItem createMenuItem(Action action, KeyStroke ks) {
        JMenuItem item = new JMenuItem(action);
        if ( ks != null ) {
            item.setAccelerator(ks);
        }
        return item;
    }
    
    public int checkExit() {
        return dialogPanel.checkExit();
    }

    

        
    
    
//  ---------------- LISTENER -------------------------------------    

    public void internalFrameOpened(InternalFrameEvent e) {
        
    }

    public void internalFrameClosing(InternalFrameEvent e) {
        close.actionPerformed(null);
    }

    public void internalFrameClosed(InternalFrameEvent e) {
        
    }

    public void internalFrameIconified(InternalFrameEvent e) {
        
    }

    public void internalFrameDeiconified(InternalFrameEvent e) {     
    }

    public void internalFrameActivated(InternalFrameEvent e) {
        
    }

    public void internalFrameDeactivated(InternalFrameEvent e) {
    }

    
    public static String _(String txt) {
        return txt;
    }

// ------------------------------------------------------
    Action close = new AbstractAction(_("Close")) {
    
        public void actionPerformed(ActionEvent e) {
            if ( dialogPanel.checkExit() != JOptionPane.CANCEL_OPTION ) {
                DialogDocument.this.setVisible(false);
                DialogDocument.this.mainEditor.closeDocument(DialogDocument.this);
            }        
        }    
    };


    public Action simpleRun = new AbstractAction(_("Test run")) {
        
        public void actionPerformed(ActionEvent e) {
            JInternalFrame preview = new JInternalFrame("Dialog preview",true,true,false,false);
            preview.setSize(500,500);
            DialogPreview dp = new DialogPreview(dialogPanel.getRoot());
            preview.getContentPane().add(dp);
            getDesktopPane().add(preview);
            preview.setVisible(true);
        }
    };
    
    public static String SHOW_PROBLEM_VIEW_STR = "Show problem view";
    public static String HIDE_PROBLEM_VIEW_STR = "Hide problem view";
    
    public Action problemCheck = new AbstractAction(_(SHOW_PROBLEM_VIEW_STR)) {
        
        public void actionPerformed(ActionEvent e) {
            dialogPanel.viewToModel();
            if ( problemFrame == null ) {
                problemFrame = new JInternalFrame("Problems pane");
                ProblemPane problemPane = dialogPanel.problemPane;
                problemFrame.getContentPane().add(problemPane);
                problemFrame.pack();
                problemFrame.setLocation(getX()+200,getY()+100);
                problemFrame.putClientProperty("JInternalFrame.isPalette",Boolean.TRUE);
                getDesktopPane().add(problemFrame,JLayeredPane.PALETTE_LAYER);
                //problemFrame.getActionMap().put("show/hide",this);
                //problemFrame.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_F5,InputEvent.CTRL_DOWN_MASK),"show/hide");
                //problemPane.getActionMap().put("show/hide",this);
                //problemPane.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_F5,InputEvent.CTRL_DOWN_MASK),"show/hide");
                problemPane.problemList.getActionMap().put("show/hide",this);
                problemPane.problemList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_F5,InputEvent.CTRL_DOWN_MASK),"show/hide");
                
            }
            toFront();
            if ( problemFrame.isVisible() ) {
                problemFrame.setVisible(false);
                putValue(Action.NAME,_(SHOW_PROBLEM_VIEW_STR));
                //getDesktopPane().setSelectedFrame(DialogDocument.this);
                
                try {
                    setSelected(true);
                } catch (PropertyVetoException e1) {
                    e1.printStackTrace();
                    // non-critical error
                }
            } else {
                dialogPanel.refreshProblems();
                problemFrame.setVisible(true);
                putValue(Action.NAME,_(HIDE_PROBLEM_VIEW_STR));
            }
            
        }
    };
    
}
