package org.gjt.abies.dialog.editor;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.util.TooManyListenersException;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.plaf.TreeUI;
import javax.swing.plaf.basic.BasicTreeUI;
import javax.swing.tree.*;

import org.gjt.abies.*;
import org.gjt.abies.Queue;
import org.gjt.abies.dialog.*;
import org.gjt.abies.dialog.Dialog;

public class DialogPanel extends JPanel implements TreeSelectionListener{
    
    public static final String DIVIDER_LOC_PROP = "divider.loc";
    public static final int MAX_UNDO_QUEUE_SIZE = 20;
    
    private Dialog root;
    File diskFile;
    FlatTree flatTree;
    DefaultTreeModel treeModel;
    DialogTreeRenderer treeRenderer;
    HierarchicalTree hierarchyTree;
    DefaultTreeModel hierarchyModel;
    JEditorPane precondition = new JEditorPane();
    JEditorPane display = new JEditorPane();
    JEditorPane exec = new JEditorPane();
    JComboBox nodeChoice = new JComboBox();
    JCheckBox autoOption = new JCheckBox();
    JTextField idField = new JTextField(10);
    TagBase currentlyEdited;
    
    Queue undoQueue = new Queue();
    Queue redoQueue = new Queue();
    JLabel statusBar = new JLabel("     ");
    private DialogPanelEnvironment env;
    ProblemPane problemPane;
    
    
    public DialogPanel(DialogPanelEnvironment anEnv, Dialog aRoot,File aFile) {
        env = anEnv;
        diskFile = aFile;
        root = aRoot;
        
        setLayout(new BorderLayout());
        
        treeModel = new DefaultTreeModel(getRoot());
        flatTree = new FlatTree(treeModel);
        treeRenderer = new DialogTreeRenderer();
        flatTree.setCellRenderer(treeRenderer);
        flatTree.setDragEnabled(true);
        
        hierarchyModel = new DefaultTreeModel(new HierarchyWrapper(null,root,root));
        hierarchyTree = new HierarchicalTree(hierarchyModel);
        hierarchyTree.setCellRenderer(treeRenderer);
        hierarchyTree.getSelectionModel().addTreeSelectionListener(this);
        
        
        final JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,true);
        split.add(createTreePane());
        split.add(createEditPane());
        split.setDividerLocation(Editor.PREFS.getInt(DIVIDER_LOC_PROP,300));
        split.addPropertyChangeListener(JSplitPane.DIVIDER_LOCATION_PROPERTY,new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                Editor.PREFS.putInt(DIVIDER_LOC_PROP,split.getDividerLocation());
            }
        });
        
        add(BorderLayout.CENTER,split);
        
        statusBar.setBorder(new BevelBorder(BevelBorder.RAISED));
        add(BorderLayout.SOUTH,statusBar);
        
        FlatTreeTransferHandler handler = new FlatTreeTransferHandler(this);
        flatTree.setTransferHandler(handler);

        
        try {
            flatTree.getDropTarget().addDropTargetListener(handler);
        } catch (TooManyListenersException e) {
            e.printStackTrace();
            // we can continue, as drag&drop is not a critical feature
        }
        
        
        flatTree.addMouseListener(new FlatTreePopup(this));
        flatTree.getSelectionModel().addTreeSelectionListener(this);
        flatTree.setExpandsSelectedPaths(true);
        
        problemPane = new ProblemPane(this);
        

        checkUndoRedo();        
    }

    
    private JComponent createTreePane() {
        JTabbedPane tabs = new JTabbedPane();
        tabs.add("Flat",new JScrollPane(flatTree));
        tabs.add("Hierarchy",new JScrollPane(hierarchyTree));
        return tabs;
    }


    private JComponent createEditPane() {
        JPanel jp = new JPanel();
        
        JPanel optionPane = new JPanel();
        optionPane.setBorder(new TitledBorder(_("Extra info")));
        JPanel idPanel = new JPanel();
        idPanel.setBorder(new TitledBorder(_("ID")));
        idPanel.add(idField);
        optionPane.add(idPanel);
        nodeChoice.setBorder(new TitledBorder(_("Forward to")));
        optionPane.add(nodeChoice);
        autoOption.setBorder(new TitledBorder(_("Auto forward")));
        optionPane.add(autoOption);
        nodeChoice.setEditable(true);
        
        nodeChoice.setEnabled(false);
        autoOption.setEnabled(false);
        nodeChoice.setModel(new DefaultComboBoxModel() {

            public int getSize() {
                return getRoot().getNodes().size();
            }

            public Object getElementAt(int index) {
                return getRoot().getNode(index).getId();
            }
        });
        
        nodeChoice.getEditor().addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                
                String str = (String)nodeChoice.getEditor().getItem();
                if ( root.containsNodeId(str)) {
                    return;
                }
                int answer = JOptionPane.showConfirmDialog(DialogPanel.this,
                        "Node " + str + " does not exists, it will be created, do you want to edit it now?",
                        "New node",JOptionPane.YES_NO_CANCEL_OPTION);
                if ( answer == JOptionPane.CANCEL_OPTION ) {
                    return;
                }
                Node node = new Node();
                node.setId(str);
                node.setParent(getRoot());
                treeModel.insertNodeInto(node,getRoot(),getRoot().getChildCount());
                updateHierarchyTree();
                if ( answer == JOptionPane.NO_OPTION ) {
                    return;
                }
                flatTree.setSelectionRow(flatTree.getRowCount()-1);
                
            }
        });
        
        
        
        idField.addFocusListener(new FocusListener() {
            
            public void focusGained(FocusEvent e) {
            }
            public void focusLost(FocusEvent e) {
                checkForIdModification();
                refreshProblems();
            }
        });
        
        idField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                checkForIdModification();
                refreshProblems();
            }
        });
        
        jp.setLayout(new GridLayout(4,1));
        
        jp.add(wrapTextDisplay(precondition,"Precondition"));
        jp.add(wrapTextDisplay(display,"Display"));
        jp.add(wrapTextDisplay(exec,"Exec"));
        jp.add(optionPane);
        return jp;
    }
    
    private JComponent wrapTextDisplay(JEditorPane edit, String name) {
        
        edit.setMinimumSize(new java.awt.Dimension(50,50));
        JScrollPane jsp = new JScrollPane(edit,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        JPanel jp = new JPanel();
        jp.setLayout(new BorderLayout());
        jp.setBorder(new TitledBorder(_(name)));
        jp.add(BorderLayout.CENTER,jsp);
        return jp;
    }
    
    public void showError(String title, String txt) {
        JOptionPane.showInternalMessageDialog(this,txt,title,JOptionPane.ERROR_MESSAGE);
    }
    
    public boolean askQuestion(String title,String txt) {
        return JOptionPane.showConfirmDialog(this,txt,title,JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION;
    }
    
    public int getLastSelectedRow() {
        int[] rows = flatTree.getSelectionRows();
        int maxRow = -1;
        if ( rows != null) {
            for (int i =0; i < rows.length; i++ ) {
                maxRow = Math.max(maxRow,rows[i]);
            }
        }
        return maxRow;
    }
    
    public void modelToView(TagBase tag) {
        if ( tag == null || tag instanceof Dialog ) {
            precondition.setText("");
            precondition.setEnabled(false);
            exec.setText("");
            exec.setEnabled(false);
            display.setText("");
            display.setEnabled(false);
            currentlyEdited = null;
            nodeChoice.setSelectedItem("");
            nodeChoice.setEnabled(false);
            autoOption.setSelected(false);
            autoOption.setEnabled(false);
            idField.setText("");
            return;
        }
        if ( tag instanceof Option ) {
            Option opt = (Option)tag;
            precondition.setText(opt.getPrecondition());
            precondition.setEnabled(true);
            exec.setText(opt.getExec());
            exec.setEnabled(true);
            display.setText(opt.getDisplay());
            display.setEnabled(true);
            currentlyEdited = opt;
            nodeChoice.setSelectedItem(opt.getForward());
            nodeChoice.setEnabled(true);
            autoOption.setSelected(opt.isAuto());
            autoOption.setEnabled(true);
            idField.setText(opt.getId());
        } else if ( tag instanceof Node ) {
            Node node = (Node)tag;
            precondition.setText("");
            precondition.setEnabled(false);
            exec.setText(node.getExec());
            exec.setEnabled(true);
            display.setText(node.getDisplay());
            display.setEnabled(true);
            currentlyEdited = node;
            nodeChoice.setSelectedItem("");
            nodeChoice.setEnabled(false);
            autoOption.setSelected(false);
            autoOption.setEnabled(false);
            idField.setText(node.getId());
        }
        //precondition.scrollRectToVisible(new Rectangle(0,0,1,1));
        //exec.scrollRectToVisible(new Rectangle(0,0,1,1));
        //display.scrollRectToVisible(new Rectangle(0,0,1,1));
    }
    
    private static boolean same(String a, String b) {
        if ( a == b ) {
            return true;
        }
        
        if ( a == null ) {
            a = "";
        }
        if ( b == null ) {
            b = "";
        }
        return a.equals(b);
    }
    
    public void viewToModel() {
        if ( currentlyEdited == null ) {
            return;
        }
        TagBase model = currentlyEdited;
        if ( model instanceof Option) {
            Option opt = (Option)model;
            boolean isTheSame = 
                same(opt.getExec(),exec.getText()) && 
                same(opt.getDisplay(),display.getText()) &&
                same(opt.getPrecondition(),precondition.getText()) &&
                same(opt.getForward(),(String)nodeChoice.getSelectedItem()) &&
                same(opt.getId(),idField.getText()) &&
                opt.isAuto() == autoOption.isSelected();
            if ( !isTheSame) {
                saveUndoPoint("Edit");
                opt.setExec(exec.getText());
                opt.setDisplay(display.getText());
                opt.setPrecondition(precondition.getText());
                opt.setForward((String)nodeChoice.getSelectedItem());
                opt.setAuto(autoOption.isSelected());
                opt.setId(idField.getText());
                updateHierarchyTree();
            }
        }
        if ( model instanceof Node) {
            Node node = (Node)model;
            boolean isTheSame = 
                same(node.getExec(),exec.getText()) && 
                same(node.getDisplay(),display.getText())&&
                same(node.getId(),idField.getText());
            if ( !isTheSame ) {
                checkForIdModification();
                saveUndoPoint("Edit");
                node.setExec(exec.getText());
                node.setDisplay(display.getText());
                node.setId(idField.getText());
                updateHierarchyTree();
            }
        }
        refreshProblems();
        
    }
    
    
    public void valueChanged(TreeSelectionEvent e) {
/*
 * If anybody will ever modify this method, please look very carefully at order of notifications
 * Generally, flat tree is a king. Editors always display what is selected in flat tree, nothing
 * if more than one thing is selected there. Any selection in flat tree will invalidate hierarchical tree
 * selection, unless it is the same single node. Any single selection in hierarchical tree will select 
 * given node in flat tree (and thus call editors update).
 * Very important thing is to NOT clear flat tree selection on hierarchical tree selection clear.
 */
        if ( e.getSource() == flatTree.getSelectionModel() ) {
            viewToModel();
    
            TreePath[] paths = flatTree.getSelectionPaths();
            if ( paths == null || paths.length == 0 || paths.length > 1) {
                modelToView(null);
                hierarchyTree.setSelectionRow(-1);
                return;
            }
            TagBase tb = (TagBase)paths[0].getLastPathComponent();
            modelToView(tb);
            setStatus(null);
            
            paths = hierarchyTree.getSelectionPaths();
            if ( paths !=null ) {
                if ( ((HierarchyWrapper)paths[0].getLastPathComponent()).node != tb) {
                    hierarchyTree.setSelectionRow(-1);
                }
            }
            
        } else {
            TreePath[] paths = hierarchyTree.getSelectionPaths();
            if ( paths != null && paths.length > 1) {
                flatTree.setSelectionRow(-1);
                return;
            }
            if (paths == null || paths.length == 0 ) {
                return;
            }
            TagBase tb = ((HierarchyWrapper)paths[0].getLastPathComponent()).node;
            selectData(new TagBase[] {tb});
            
        }
    }
    
    public int checkExit() {
        
        viewToModel();

        
        String question;
        
        if ( diskFile == null ) {
            question = "File "+getEnv().getTitle()+" not saved, do you want to save before closing it ?";
        } else {
            try {
                BufferedReader br = new BufferedReader(new FileReader(diskFile));
                StringBuffer sb = new StringBuffer();
                while ( true ) {
                    String line = br.readLine();
                    if ( line == null ) {
                        break;
                    }
                    sb.append(line +"\n");
                }
                br.close();
                String file = StringUtil.replace(sb.toString(),"\r","").trim();
                String rootStr = StringUtil.replace(getRoot().toString(),"\r","").trim();
                if ( file.equals(rootStr)) {
                    return JOptionPane.YES_OPTION;
                }
                question = "Dialog "+getEnv().getTitle()+" has been changed, do you want to save it ?";
                    
            } catch (IOException exc) {
                exc.printStackTrace();
                question = "Problem when comparing current edit to saved version for "+getEnv().getTitle()+", do you want to save it to be sure ?";
            }
            
        }
        getEnv().toFront();
        int answer = JOptionPane.showConfirmDialog(this,question,"Unsaved changes for " + getEnv().getTitle(),JOptionPane.YES_NO_CANCEL_OPTION);
        if ( answer == JOptionPane.CANCEL_OPTION ) {
            return JOptionPane.CANCEL_OPTION;
        } else if ( answer == JOptionPane.NO_OPTION ) {
            return JOptionPane.NO_OPTION ;
        } else { //YES_OPTION
            save.actionPerformed(null);
            return JOptionPane.YES_OPTION;
        }
        
    }
    
    public File showSaveDialog(File proposedFile) {
        
        JFileChooser jfc = new JFileChooser();
        if ( diskFile != null ) {
            jfc.setSelectedFile(proposedFile);
        } else {
            jfc.setCurrentDirectory(new File(Editor.PREFS.get(Editor.OPEN_DIR,".")));
        }
        int result = jfc.showSaveDialog(jfc);
        if (result != JFileChooser.APPROVE_OPTION) {
            return null;
        }
        File currentDir = jfc.getCurrentDirectory();
        
        File f = jfc.getSelectedFile();
        
        if ( f.exists() ) {
            if ( !askQuestion("Overwrite","Do you want to overwrite " + f + " ?") ) {
                return null;
            }
        }
        
        Editor.PREFS.put(Editor.OPEN_DIR,currentDir.getAbsolutePath());
        
        return f;
    }

    
    
    
    public void saveUndoPoint(String action) {
        
        while ( undoQueue.size() >= MAX_UNDO_QUEUE_SIZE) {
            undoQueue.removeTail();
        }
        undoQueue.addHead(getRoot().deepClone());
        
        // clear redo queue
        while ( redoQueue.size() > 0 ) {
            redoQueue.removeHead();
        }
        
        //TODO: add action description to queue
        checkUndoRedo();
    }
    
    public void saveRedoPoint(String action) {
        
        while ( redoQueue.size() >= MAX_UNDO_QUEUE_SIZE) {
            redoQueue.removeTail();
        }
        redoQueue.addHead(getRoot().deepClone());
        
        //TODO: add action description to queue
        checkUndoRedo();
    }
    
    public void checkUndoRedo() {
        undo.setEnabled(undoQueue.size() > 0);
        redo.setEnabled(redoQueue.size() > 0);
    }
    
    public void setStatus(String txt) {
        if ( txt == null ) {
            txt = "      ";
        }
        statusBar.setText(txt);
    }
    
    
    public void setEnv(DialogPanelEnvironment env) {
        this.env = env;
    }

    public DialogPanelEnvironment getEnv() {
        return env;
    }

    public void setRoot(Dialog root) {
        this.root = root;
        treeModel.setRoot(root);
        updateHierarchyTree();
    }

    public Dialog getRoot() {
        return root;
    }
    
    boolean askingForIdUpdate;
    
    public void checkForIdModification() {
        if ( currentlyEdited == null ) {
            return;
        }
        if ( !(currentlyEdited instanceof Node )) {
            return;
        }
        if ( currentlyEdited.getId().equals(idField.getText())) {
            return;
        }
        if ( askingForIdUpdate ) {
            return;
        }
        
        askingForIdUpdate = true;

        int answer = JOptionPane.showConfirmDialog(this,"You have changed id of node, do you want to target all options pointing to it to new id ?",
                "Retarget nodes",JOptionPane.YES_NO_CANCEL_OPTION);
        
        askingForIdUpdate = false;
        if ( answer == JOptionPane.CANCEL_OPTION) {
            idField.setText(currentlyEdited.getId());
            return;
        }
        
        
        Node node = (Node)currentlyEdited;
        String oldId = node.getId();
        String newId = idField.getText();
        node.setId(newId);
        
        treeModel.nodeChanged(node);
        updateHierarchyTree();
        
        if ( answer == JOptionPane.NO_OPTION ) {
            return;
        }
        
        for ( int i =0; i < root.getNodes().size(); i++) {
            Node n = root.getNode(i);
            for ( int j =0; j < n.getOptions().size(); j++) {
                Option opt = n.getOption(j);
                if ( oldId.equalsIgnoreCase(opt.getForward()) ) {
                    opt.setForward(newId);
                    treeModel.nodeChanged(opt);
                    updateHierarchyTree();
                }
            }
        }
        
    }
    
    
    public void selectData(TagBase[] bases) {
        TreePath[] toSelect = new TreePath[bases.length];
        for ( int i =0; i < toSelect.length; i++ ) {
            toSelect[i] = new TreePath(treeModel.getPathToRoot(bases[i]));
        }
        flatTree.setSelectionPaths(toSelect);
    }

    
    public void refreshProblems() {
        java.util.List problems = getRoot().performProblemCheck();
        problemPane.setProblems(problems);
    }
    
    public void updateHierarchyTree() {
        hierarchyModel.setRoot(new HierarchyWrapper(null,root,root));
    }
    
    public static String _(String txt) {
        return txt;
    }

//  ---------------- ACTIONS -------------------------------------    
    
    
    

    public Action save = new AbstractAction(_("Save")) {
        public void actionPerformed(ActionEvent e) {
            if ( diskFile == null ) {
                saveAs.actionPerformed(e);
                return;
            }
            try {
                FileWriter fw = new FileWriter(diskFile);
                // set entry point to first node in tree
                if ( getRoot().getNodes().size() > 0 ) {
                    getRoot().setEntry(getRoot().getNode(0).getId());
                }
                fw.write(getRoot().toString());
                fw.close();
                setStatus("File saved");
            } catch (IOException exc) {
                showError("Error while saving " + diskFile, exc.getMessage());
            }
            
        }
    };
    
    public Action saveAs = new AbstractAction(_("Save as..")) {
        public void actionPerformed(ActionEvent e) {
            File toSave = DialogPanel.this.showSaveDialog(diskFile);
            if ( toSave == null ) {
                return;
            }
            diskFile = toSave;
            getEnv().setTitle(diskFile.getName());
            save.actionPerformed(e);
        }
    };
    
    
    
    public Action addNode = new AbstractAction(_("Add node")) {
        
        public void actionPerformed(ActionEvent e) {
            int row = getLastSelectedRow();
            if ( row < 0 ) {
                return;
            }
            TreeNode tb = (TagBase)flatTree.getPathForRow(row).getLastPathComponent();
            if ( tb == null ) {
                return;
            }
            
            saveUndoPoint("add node");
            
            if ( tb instanceof Option ) {
                tb = tb.getParent();
            }
            int after = getRoot().getIndex(tb)+1;
            
            Node node = new Node();
            
            int id = getRoot().getNodes().size();
            while ( true ) {
                String testId = "N"+id;
                if ( !getRoot().containsNodeId(testId)) {
                    node.setId(testId);
                    break;
                }
                id++;
            }
            node.setParent(getRoot());
            treeModel.insertNodeInto(node,getRoot(),after);
            updateHierarchyTree();
            flatTree.expandRow(row);
            flatTree.setSelectionRow(row+1);
        }
        
    };
    
    public Action addOption = new AbstractAction(_("Add option")) {
        
        public void actionPerformed(ActionEvent e) {
            int row = getLastSelectedRow();
            if ( row < 0 ) {
                return;
            }
            TreeNode tb = (TagBase)flatTree.getPathForRow(row).getLastPathComponent();
            if ( tb == null || (tb instanceof Dialog)) {
                return;
            }
            
            saveUndoPoint("add option");
            
            Node node;
            if ( tb instanceof Option ) {
                node = (Node)tb.getParent();
            } else {
                node = (Node)tb;
            }
            int after = node.getIndex(tb)+1;
            Option opt = new Option();
            
            int id = node.getOptions().size();
            while ( true ) {
                String testId = "A"+id;
                if ( !node.containsOptionId(testId)) {
                    opt.setId(testId);
                    break;
                }
                id++;
            }
            
            opt.setForward("EXIT");
            opt.setParent(node);
            
            treeModel.insertNodeInto(opt,node,after);
            updateHierarchyTree();
            flatTree.expandRow(row);
            flatTree.setSelectionRow(row+1);
        }
        
    };
    


    public Action exportAsJava = new AbstractAction(_("Sample export as java..")) {
        
        public void actionPerformed(ActionEvent e) {
            DialogPanel.this.viewToModel();
            File proposed = null;
            if ( diskFile != null ) {
                if ( diskFile.getPath().indexOf(".") > 0 ) {
                    proposed = new File( diskFile.getPath().substring(0,diskFile.getPath().indexOf(".")) +".java");
                } else {
                    proposed = new File( diskFile.getPath()+".java");
                }
                
            }
            File toSave = DialogPanel.this.showSaveDialog(proposed);
            if ( toSave == null ) {
                return;
            }
            
            String name = toSave.getName();
            String javaName = name.substring(0,name.length()-".java".length());
            DialogToJava dtj = new DialogToJava();
            dtj.dialog = getRoot();
            try {
                dtj.generateJavaFile(javaName,toSave);
                setStatus("File exported");
            } catch (IOException exc) {
                showError("Error while exporting", exc.getMessage());
            }
            
            
        }
        
    };
    
    public Action deleteSelected = new AbstractAction(_("Delete selected")) {
        public void actionPerformed(ActionEvent e) {
            
            TreePath[] paths = flatTree.getSelectionPaths();
            if ( paths != null ) {
                saveUndoPoint("delete selected");
                for ( int i =0; i < paths.length; i++ ) {
                    MutableTreeNode mtn = (MutableTreeNode)paths[i].getLastPathComponent();
                    if ( mtn.getParent() != null ) {
                        treeModel.removeNodeFromParent(mtn);
                        updateHierarchyTree();
                    }
                }
                setStatus("Deleted entries");
            }
            
        }
    };
    
    public Action undo = new AbstractAction(_("Undo")) {
        public void actionPerformed(ActionEvent e) {
            if ( undoQueue.size() == 0 ) {
                return;
            }
            saveRedoPoint(null);
            setRoot((Dialog)undoQueue.removeHead());
            
            //TODO: set status to undone action description
            checkUndoRedo();
            setStatus("Undone level " + (undoQueue.size()+1));
        }
    };
    
    public Action redo = new AbstractAction(_("Redo")) {
        public void actionPerformed(ActionEvent e) {
            if ( redoQueue.size() == 0 ) {
                return;
            }
            
            while ( undoQueue.size() >= MAX_UNDO_QUEUE_SIZE) {
                undoQueue.removeTail();
            }
            undoQueue.addHead(getRoot().deepClone());
            

            setRoot((Dialog)redoQueue.removeHead());
            //TODO: set status to undone action description 
            checkUndoRedo();
            setStatus("Redone level " + undoQueue.size());
        }
    };
    
    public Action refresh = new AbstractAction(_("Refresh")) {
        
        public void actionPerformed(ActionEvent e) {
            // terrible hack to workaround non-updating node widths
            TreeUI ui = flatTree.getUI();
            if ( !(ui instanceof BasicTreeUI ) ) {
                return;
            }
            BasicTreeUI btui = (BasicTreeUI)ui;
            btui.setLeftChildIndent(btui.getLeftChildIndent());
        }
    };


}
