package org.gjt.abies.dialog.editor;

import java.awt.*;
import java.awt.event.*;
import java.awt.event.MouseListener;
import java.util.List;

import javax.swing.*;
import javax.swing.text.*;

import org.gjt.abies.dialog.*;
import org.gjt.abies.dialog.Dialog;

public class DialogPreview extends JPanel implements MouseListener {
    
    Dialog root;
    DefaultStyledDocument doc = new DefaultStyledDocument();
    DefaultListModel optionsModel = new DefaultListModel();
    JEditorPane mainText = new JEditorPane();
    JList options = new JList(optionsModel);
    
    
    public DialogPreview(Dialog aRoot) {
        root = aRoot;
        mainText.setContentType("text/rtf");
        mainText.setDocument(doc);
        setLayout(new BorderLayout());
        add(BorderLayout.CENTER,new JScrollPane(mainText));
        add(BorderLayout.SOUTH,new JScrollPane(options));
        options.setCellRenderer(new DefaultListCellRenderer() {
            public Component getListCellRendererComponent(JList list,
                    Object value, int index, boolean isSelected,
                    boolean cellHasFocus) {
                Option opt = (Option)value;
                String str = (index+1) + ". " + opt.getDisplay();
                if ( opt.getPrecondition() != null ) {
                    str += " ["+opt.getPrecondition() + "]";
                }
                if ( opt.isAuto() ) {
                    str = "[AUTO]" + str;
                }
                return super.getListCellRendererComponent(list, str, index,
                        isSelected, cellHasFocus);
            }
        });
        displayNode(root.getNode(0));
        options.addMouseListener(this);
        for ( int i = 1; i <=9; i++ ) {
            final int index = i;
            getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_0+i,0),"pressed-"+i);
            getActionMap().put("pressed-"+i,new AbstractAction("Pressed " + i) {
                public void actionPerformed(ActionEvent e) {
                    if ( optionsModel.getSize() < index ) {
                        Toolkit.getDefaultToolkit().beep();
                        return;
                    }
                    Option opt = (Option)optionsModel.get(index-1);
                    selectedOption(opt);
                }
            });
        }
    }
    
    public AttributeSet getColor(Color c) {
        MutableAttributeSet mas = new SimpleAttributeSet();
        StyleConstants.setForeground(mas,c);
        return mas;
    }
    
    public void addString(String str, Color c) {
        try {
            doc.insertString(doc.getLength(),str,getColor(c));
        } catch (BadLocationException e) {
            e.printStackTrace();
        }
        mainText.scrollRectToVisible(new Rectangle(0,mainText.getHeight()-1,1,1));
    }
    
    public void displayNode(Node node) {
        
    
        if ( node.getExec() != null ) {
            addString(""+node.getExec() + "\n",Color.GREEN);
        }
        addString(node.getDisplay() + "\n",Color.BLACK);
    
        
        displayOptions(node.getOptions());
    }
    
    public void displayOptions(List allOptions) {
        optionsModel.removeAllElements();
        for ( int i =0; i < allOptions.size(); i++ ) {
            optionsModel.addElement(allOptions.get(i));
        }
    }
    
    public void displaySelectedOption(Option opt) {

        addString(opt.getDisplay() + "\n",Color.BLUE);
        if ( opt.getExec() != null ) {
            addString(opt.getExec() + "\n",Color.GREEN);
        }

    }
    
    public void selectedOption(Option opt) {
        displaySelectedOption(opt);
        String forward = opt.getForward();
        Node node = root.findNode(forward);
        if ( node == null ) {
            addString("Missing forward " + forward + " for option " + opt.getId() + "\n", Color.RED);
        } else {
            displayNode(node);
        }
    }

    public void mouseClicked(MouseEvent e) {

        Option opt = (Option)options.getSelectedValue();
        if ( opt == null ) {
            return;
        }
        selectedOption(opt);
    }

    public void mouseEntered(MouseEvent e) {
   
    }

    public void mouseExited(MouseEvent e) {
    
    }

    public void mousePressed(MouseEvent e) {
       
    }

    public void mouseReleased(MouseEvent e) {
      
    }
    
    

}
