package org.gjt.abies.dialog.editor;

import java.awt.Rectangle;
import java.awt.event.*;
import java.io.*;
import java.util.ArrayList;
import java.util.prefs.Preferences;

import javax.swing.*;

import org.gjt.abies.dialog.*;




public class Editor extends JFrame implements WindowListener {
    
    public static final Preferences PREFS = Preferences.userNodeForPackage(Editor.class);
    
    public static final String OPEN_DIR = "open.dir";
    public static final String MAIN_FRAME_BOUNDS = "main.frame.bounds";
    
    JDesktopPane desktop;
    ArrayList openDocuments = new ArrayList();
    
    public Editor() {

        super("test");
        setBounds(getPrefBounds(MAIN_FRAME_BOUNDS,"0x0x800x800"));
        addComponentListener(new ResizeListener(MAIN_FRAME_BOUNDS));

        desktop = new JDesktopPane();
        getContentPane().add(desktop);

        
        JMenuBar jmb = new JMenuBar();
            JMenu file = new JMenu(_("File"));
                file.add(newAction);
                file.add(createMenuItem(openAction,KeyStroke.getKeyStroke(KeyEvent.VK_O,InputEvent.CTRL_DOWN_MASK)));
                file.addSeparator();
                file.add(createMenuItem(saveAll,KeyStroke.getKeyStroke(KeyEvent.VK_S,InputEvent.CTRL_DOWN_MASK|InputEvent.SHIFT_DOWN_MASK)));
                file.add(createMenuItem(closeAll,KeyStroke.getKeyStroke(KeyEvent.VK_F4,InputEvent.CTRL_DOWN_MASK|InputEvent.SHIFT_DOWN_MASK)));
                file.addSeparator();
                file.add(exit);
            jmb.add(file);
            JMenu help = new JMenu(_("Help"));
                help.add(new JMenuItem(about));
            jmb.add(help);
        setJMenuBar(jmb);
        
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(this);
    }

    public void windowOpened(WindowEvent e) {
        
    }

    public void windowClosing(WindowEvent e) {
        exit.actionPerformed(null);
    }

    public void windowClosed(WindowEvent e) {
        
    }

    public void windowIconified(WindowEvent e) {
        
    }

    public void windowDeiconified(WindowEvent e) {
        
    }

    public void windowActivated(WindowEvent e) {
        
    }

    public void windowDeactivated(WindowEvent e) {
        
    }
    
    Action newAction = new AbstractAction(_("New dialog")) {
        public void actionPerformed(ActionEvent e) {
            Dialog dialog = new Dialog();
            Node start = new Node();
            start.setId("START");
            start.setDisplay("Dialog started");
            start.setParent(dialog);
            dialog.setEntry(start.getId());
            dialog.getNodes().add(start);
            
            DialogDocument dd = openDocument(dialog,null);
            desktop.add(dd);
            dd.setVisible(true);
        }
    };
    
	

    Action openAction = new AbstractAction(_("Open..")) {
        
        public void actionPerformed(ActionEvent e) {
            String currentStr = PREFS.get(OPEN_DIR,".");
            JFileChooser jfc = new JFileChooser(currentStr);
            int result = jfc.showOpenDialog(Editor.this);
            if (result != JFileChooser.APPROVE_OPTION) {
                return;
            }
            File currentDir = jfc.getCurrentDirectory();
            PREFS.put(OPEN_DIR,currentDir.getAbsolutePath());
            File f = jfc.getSelectedFile();
            DialogParser dp = new DialogParser();
            try {
                Reader r = new BufferedReader(new FileReader(f));
                dp.parseDialog(r);
                r.close();
            } catch (Exception exc) {
                JOptionPane.showMessageDialog(Editor.this,exc.getMessage(),"Error while loading " + f.getName(),JOptionPane.ERROR_MESSAGE);
                exc.printStackTrace();
                return;
            }
            DialogDocument dd = openDocument(dp.dialog,f);
            desktop.add(dd);
            dd.setVisible(true);
        }
    };
    
    Action closeAll = new AbstractAction(_("Close all") ) {
      
        public void actionPerformed(ActionEvent e) {
            ArrayList docCopy = new ArrayList();
            docCopy.addAll(Editor.this.openDocuments);
            for ( int i =0; i < docCopy.size(); i++ ) {
                DialogDocument dd = (DialogDocument)docCopy.get(i);
                int answer = dd.checkExit();
                if ( answer == JOptionPane.CANCEL_OPTION ) {
                    return;
                }
                closeDocument(dd);
                dd.setVisible(false);
            }
        }  
    };
    
    Action saveAll = new AbstractAction(_("Save all") ) {
        
          public void actionPerformed(ActionEvent e) {
              ArrayList docCopy = new ArrayList();
              docCopy.addAll(Editor.this.openDocuments);
              for ( int i =0; i < docCopy.size(); i++ ) {
                  DialogDocument dd = (DialogDocument)docCopy.get(i);
                  dd.dialogPanel.save.actionPerformed(e);
              }
          }  
      };
    
    Action exit = new AbstractAction(_("Exit") ) {
        
          public void actionPerformed(ActionEvent e) {
              closeAll.actionPerformed(e);
              if ( openDocuments.size() == 0 ) {
                  System.exit(0);
              }
          }  
      };
    
    Action about = new AbstractAction(_("About")) {
        public void actionPerformed(ActionEvent e) {
            JOptionPane.showMessageDialog(Editor.this,ABOUT_STRING,"About RPG Dialog Editor",JOptionPane.PLAIN_MESSAGE);
        }
    };
    
    
    protected DialogDocument openDocument(Dialog dialog, File f) {
        DialogDocument dd = new DialogDocument(dialog,f,Editor.this);
        openDocuments.add(dd);
        return dd;
    }
    
    public void closeDocument(DialogDocument dd) {
        openDocuments.remove(dd);
    }
    
    public static Rectangle getPrefBounds(String id, String val) {
        //TODO: recognize maximized state and do not remember it
        val = PREFS.get(id,val);
        
        String[] posSpl = val.split("x");

        return new Rectangle(Integer.parseInt(posSpl[0]), Integer.parseInt(posSpl[1]),
            Integer.parseInt(posSpl[2]), Integer.parseInt(posSpl[3]));
    }
    
    public static void setPrefBounds(String id, Rectangle bounds) {
        PREFS.put(id,bounds.x+"x"+bounds.y+"x"+bounds.width+"x"+bounds.height);
    }
    
    private static JMenuItem createMenuItem(Action action, KeyStroke ks) {
        JMenuItem item = new JMenuItem(action);
        if ( ks != null ) {
            item.setAccelerator(ks);
        }
        return item;
    }
    
    
    public static String _(String txt) {
    	return txt;
    }

    
    public static void main(String[] args) throws Exception {
        Editor editor = new Editor();
        editor.setVisible(true);
        editor.openAction.actionPerformed(null);
    }
    
    public static final String ABOUT_STRING = 
        "<html><h1>RPG Dialog Maker</h1>" +
        "Author: Artur Biesiadowski<br>Version: 13 April 2004, 23:30<br>" +
        "</html>";
}
