package org.gjt.abies.dialog.editor;

import java.awt.datatransfer.*;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.*;
import java.awt.dnd.DropTargetListener;
import java.io.*;
import java.io.IOException;
import java.util.*;
import java.util.ArrayList;

import javax.swing.*;
import javax.swing.TransferHandler;
import javax.swing.tree.*;
import javax.swing.tree.TreePath;

import org.gjt.abies.XmlOutput;
import org.gjt.abies.dialog.*;
import org.gjt.abies.nanogroovy.*;



public class FlatTreeTransferHandler extends TransferHandler implements Comparator, DropTargetListener {
    
    public static final String NODE_LIST = "nodelist";
    public static final String OPTION_LIST = "optionlist";
    
    FlatTree tree;
    DefaultTreeModel treeModel;
    DialogPanel doc;
    
    public FlatTreeTransferHandler(DialogPanel aDoc) {
        doc = aDoc;
    	tree = doc.flatTree;
        treeModel = doc.treeModel;
    }
    
    private void checkComponent(JComponent c) {
        //if ( c != tree) {
        //    throw new IllegalArgumentException("This transfer handler was initialized for different component");
        //}
    }
    
    protected Transferable createTransferable(JComponent c) {
    	checkComponent(c);
        
        TreePath[] paths = tree.getSelectionPaths();
        
        if (paths == null || paths.length == 0) {
            return null;
        }
        
        TreeModel model = tree.getModel();
        TreePath lastPath = null;
        TreePath[] displayPaths = getDisplayOrderPaths(paths);
        
        boolean optionsOnly = true;

        for (int i = 0; i < displayPaths.length; i++) {
            
            TreePath path = displayPaths[i];
            Object node = path.getLastPathComponent();
            
            if ( node instanceof Dialog ) {
                return new PlainTransferable(node.toString(),new ArrayList());
            }
            
            
            if ( ! (node instanceof Option) ) {
            	optionsOnly = false;
            }
            
        }
        
        ArrayList toRemove = new ArrayList();
        
        //TODO: handle selected DIALOG node
        //TODO: fix mixed-select logic
        XmlOutput xml = new XmlOutput();
        if ( optionsOnly ) {
        	xml.start(OPTION_LIST);
            for (int i = 0; i < displayPaths.length; i++) {
                TreePath path = displayPaths[i];
                TagBase node = (TagBase)path.getLastPathComponent();
                node.dumpXml(xml);
                toRemove.add(node);
            }
            xml.end();
        } else {
            xml.start(NODE_LIST);
            for (int i = 0; i < displayPaths.length; i++) {
                TreePath path = displayPaths[i];
                TagBase node = (TagBase)path.getLastPathComponent();
                if ( node instanceof Node ) {
                	node.dumpXml(xml);
                    toRemove.add(node);
                } else {
                    return null;
                }
            }
            xml.end();
        }
        return new PlainTransferable(xml.dumpXML(),toRemove);
    }
    
    protected void exportDone(JComponent source, Transferable data, int action) {
        doc.saveUndoPoint("export done");
        if ( action == MOVE ) {
        	if ( data instanceof PlainTransferable ) {
        		List toRemove = ((PlainTransferable)data).toRemove;
                Iterator it = toRemove.iterator();
                while ( it.hasNext() ) {
                    MutableTreeNode mtn = (MutableTreeNode)it.next();
                    if ( mtn.getParent() != null ) {
                    	treeModel.removeNodeFromParent(mtn);
                    }
                }
            }
        }
    }



    TreePath[] getDisplayOrderPaths(TreePath[] paths) {
        // sort the paths to display order rather than selection order
        ArrayList selOrder = new ArrayList();
        for (int i = 0; i < paths.length; i++) {
            selOrder.add(paths[i]);
        }
        Collections.sort(selOrder, this);
        int n = selOrder.size();
        TreePath[] displayPaths = new TreePath[n];
        for (int i = 0; i < n; i++) {
            displayPaths[i] = (TreePath) selOrder.get(i);
        }
        return displayPaths;
    }
    
    public int compare(Object o1, Object o2) {
        int row1 = tree.getRowForPath((TreePath)o1);
        int row2 = tree.getRowForPath((TreePath)o2);
        return row1 - row2;
    }
    

    
    
	public boolean importData(JComponent comp, Transferable t) {
        checkComponent(comp);
        doc.saveUndoPoint("import data");
        try {
            XMLElement xml = new XMLElement();
            xml.parseFromReader(DataFlavor.getTextPlainUnicodeFlavor().getReaderForText(t));
            boolean optionsOnly;
            if ( OPTION_LIST.equalsIgnoreCase(xml.getName()) ) {
                optionsOnly = true;
            } else if ( NODE_LIST.equalsIgnoreCase(xml.getName()) ){
                optionsOnly = false;
            } else if ("dialog".equalsIgnoreCase(xml.getName()) ) {
                optionsOnly = false;
            } else {
                return false;
            }

            TagBase tb = (TagBase)tree.getSelectionPath().getLastPathComponent();
            if ( tb == null ) {
                return false;
            }
            
            
            if ( optionsOnly) {
                Option[] opts = new Option[xml.countChildren()];
                ArrayList children = xml.getChildren();
                for ( int i =0; i < opts.length; i++ ) {
                    opts[i] = new Option();
                    opts[i].parse((XMLElement)children.get(i),null);
                }
                Node parent;
                int start;
                if ( tb instanceof Node ) {
                    parent = (Node)tb;
                    start = 0;
                } else if ( tb instanceof Option ) {
                    parent = (Node)(((Option)tb).getParent());
                    start = parent.getIndex(tb)+1;
                } else {
                    return false;
                }
                
                for ( int i =0; i < opts.length; i++ ) {
                    opts[i].setParent(parent);
                    treeModel.insertNodeInto(opts[i],parent,start+i);
                }
                
            } else {
                Node[] nodes = new Node[xml.countChildren()];
                ArrayList children = xml.getChildren();
                for ( int i =0; i < nodes.length; i++ ) {
                    nodes[i] = new Node();
                    nodes[i].parse((XMLElement)children.get(i),null);
                }
                Dialog parent;
                int start;
                
                if ( tb instanceof Node ) {
                    parent = (Dialog)tb.getParent();
                    start = parent.getIndex((Node)tb) + 1;
                } else if ( tb instanceof Dialog ) {
                    parent = (Dialog)tb;
                    start = 0;
                } else {
                    return false;
                }
                
                for ( int i =0; i < nodes.length; i++ ) {
                    nodes[i].setParent(parent);
                    treeModel.insertNodeInto(nodes[i],parent,start+i);
                }
            }
        	
        } catch (Exception exc) {
        	doc.showError("Error during import", exc.toString());
            exc.printStackTrace();
        }
        
        return true;
	}
    
    
	public int getSourceActions(JComponent c) {
		return COPY_OR_MOVE;
	}
    
    public boolean canImport(JComponent comp, DataFlavor[] transferFlavors) {
        checkComponent(comp);
		return true;
	}
    
    
    static class PlainTransferable implements Transferable {

        static final DataFlavor PLAIN_FLAVOR = 
            //new DataFlavor("text/plain;charset=utf8;class=java.io.InputStream", "Plain Text");
            DataFlavor.getTextPlainUnicodeFlavor();
        
        String txt;
        List toRemove;
    	public PlainTransferable(String aTxt, List remove) {
            txt = aTxt;
            toRemove = remove;
        }
        
		public DataFlavor[] getTransferDataFlavors() {
			return new DataFlavor[] {PLAIN_FLAVOR};
		}
		public boolean isDataFlavorSupported(DataFlavor flavor) {
			return flavor.equals(PLAIN_FLAVOR);
		}
		public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException {
			return new ByteArrayInputStream(txt.getBytes(PLAIN_FLAVOR.getParameter("charset")));
		}
    }


    public void dragEnter(DropTargetDragEvent dtde) {
        doc.getEnv().toFront();
    }

    public void dragOver(DropTargetDragEvent dtde) {
        TreePath tp = tree.getSelectionPath();
        if ( tp == null ) {
            dtde.rejectDrag();
            return;
        }
        Object obj = tp.getLastPathComponent();
        
        XMLElement xml = new XMLElement();
        try {
            Transferable trans = null;
            //1.4
            try {
                DropTargetContext context = dtde.getDropTargetContext();
                java.lang.reflect.Method getTransferableMethod = context.getClass().getDeclaredMethod("getTransferable", new Class[0]);
                getTransferableMethod.setAccessible(true);
                trans = (Transferable)getTransferableMethod.invoke(context, new Object[0]);
            } catch (Exception exc) {
                exc.printStackTrace();
            }
            
            /*1.5
             * trans = dtde.getTransferable();
             */
            
            if ( trans == null ) {
                return;
            }
            
            xml.parseFromReader(DataFlavor.getTextPlainUnicodeFlavor().getReaderForText(trans));
            boolean optionsOnly;
            if ( OPTION_LIST.equalsIgnoreCase(xml.getName()) ) {
                if (!(obj instanceof Node || obj instanceof Option)) {
                    dtde.rejectDrag();
                }
            } else if ( NODE_LIST.equalsIgnoreCase(xml.getName()) ){
                if ( !(obj instanceof Node || obj instanceof Dialog)) {
                    dtde.rejectDrag();
                }
            } else if ("dialog".equalsIgnoreCase(xml.getName()) ) {
                if ( !(obj instanceof Node || obj instanceof Dialog)) {
                    dtde.rejectDrag();
                }
            } else {
                dtde.rejectDrag();
            }
        } catch (Exception exc) {
            exc.printStackTrace();
            dtde.rejectDrag();
        }
        
    }

    public void dropActionChanged(DropTargetDragEvent dtde) {
        
    }

    public void dragExit(DropTargetEvent dte) {
        
    }

    public void drop(DropTargetDropEvent dtde) {
        
    }
    
    
    
}
